<?php
// This file is part of DENIF, distributed under license GPLv3.
// Copyright 2006 Guillaume Ryder, Arthur Charguraud

require_once 'shared.inc.php';


class PageFormDelete extends PageDenif
{
	public $bDelete, $iObject, $id, $sConfirmURL, $sCancelURL;
	
	
	protected function getParameters() { return concatLists(parent::getParameters(), 'bDelete;iObject;id;sConfirmURL;sCancelURL'); }
	
	protected function getTitle() { return 'Confirmation de suppression'; }
	
	
	protected function initialize()
	{
		parent::initialize();
		$this->wantEditing(true);
		$this->loadParameters();
	}
	
	
	protected function contents()
	{
		$iObject = $this->iObject;
		$id      = $this->id;
		$info    = ObjectTypeInfo::get($iObject);
		if (!$info)
			$this->fatalError('Type dobjet invalide.');
		
		// Get the object to delete, check there is one and only one
		$sTable = $info->table->sName;
		$obj = sql::selectOneFetch($sTable, array('*'), sql::whereId($id), $sTable);
		if (!$obj)
			$this->fatalError('Objet introuvable.');
		if ($obj instanceof node)
			$obj->loadAllParents(true);
		
		if (!$this->sessionManager->hasRightDelete($obj))
			$this->fatalError('Vous navez pas le droit de supprimer cet objet.');
		
		if (!$obj->canDelete())
			$this->fatalError('Impossible de supprimer cet objet.');
		
		if ($this->bDelete) {
			$sError = $obj->delete();
			if (!$sError)
				self::redirect($this->targetFromURL($this->sConfirmURL)->url());
		}else
			$sError = '';
		
		$s = '';
		
		$aButton = array();
		if ($sError) {
			$s .= html::par($sError, attr::className('error'));
		}else{
			$s .= html::par('Voulez-vous vraiment supprimer cette entre&nbsp;?');
			$s .= html::par($obj->dumpBeforeDelete());
			$aButton[] = new DenifLink($this->getSelfTarget(targetSession, array(
					'bDelete'     => true,
					'iObject'     => $iObject,
					'id'          => $id,
					'sConfirmURL' => $this->sConfirmURL,
				)), 'Confirmer la suppression');
		}
		$aButton[] = new DenifLink(target::fromURL($this->sCancelURL), 'Annuler');
		
		$s .= $this->linkEditBar($aButton);
		
		return $s;
	}
}

Page::output(new PageFormDelete);

?>