<?php
// This file is part of DENIF, distributed under license GPLv3.
// Copyright 2006 Guillaume Ryder, Arthur Charguraud

require_once 'shared.inc.php';


class PageFormEditDocs extends PageDenif
{
	public $id, $bPaneLeft, $sExitURL;
	protected $node = null, $aParent, $aAncestor, $iWebMode;
	
	
	protected function getParameters() { return concatLists(parent::getParameters(), 'id;bPaneLeft;sExitURL'); }
	
	protected function isFrameset() { return $this->node && !$this->bPaneLeft; }
	
	protected function getTitle()
	{
		return 'diter une liste de documents';
	}
	
	
	protected function getCSS() { return sRootRel . 'edit_docsleft.css'; }
	
	
	protected function initialize()
	{
		parent::initialize();
		$this->loadParameters();
		
		if (!$this->wantEditing(false))
			throw new PageException;
		
		$this->iWebMode = $this->sessionManager->getWebMode();
		
		$node = $this->node = getNode($this->id);
		if (!$node)
			$this->fatalError('Objet introuvable.');
		if ($node->iNode != nodeDocGroup)
			$this->fatalError('Type dobjet invalide.');
		if (!$this->sessionManager->hasRightModify($node))
			$this->fatalError('Vous navez pas lautorisation de modifier cet objet.');
		
		$this->aParent     = $node->loadAllParents(true);
		$this->aAncestor   = $this->aParent;
		$this->aAncestor[] = $node;
		
		// Process AJAX request
		$sAction = $_REQUEST->getSafe('sSave');
		if ($sAction)
			$this->processCommand($node, JSON::inputSafe($sAction));
	}
	
	
	protected function scripts()
	{
		$s = parent::scripts();
		
		$node = $this->node;
		if ($node && !$this->isFrameset()) {
			$sScript = '';
			
			$sScript .= html::jscriptSetVar('sSession', $this->sessionManager->getCode());
			
			// Give humans list in the 'humans' JScript array
			$aHumanScript = array();
			foreach (getProperties(propertyHuman) as $human)
				$aHumanScript[$human->id] = $human->sName;
			$sScript .= html::jscriptSetVar('humans', $aHumanScript);
			
			// Retreive humans, put them in document
			$aDocument = bijectiveBy($node->loadChildren(), 'id');
			foreach ($aDocument as $document)
				$document->aidHuman = array();
			foreach ($node->getLinksRecurse(nodeDocGroup, nodeDocument) as $link)
				if ($link->iProperty == propertyHuman && isset($aDocument[$link->idNode]))
					$aDocument[$link->idNode]->aidHuman[$aHumanScript[$link->idProperty]] = $link->idProperty;
			
			// Give documents data in the 'docs' JScript array:
			// [id, sTitle, sURL, [flag1, ..., flagN], [idHuman1, ..., idHumanN]]
			$aDocumentScript = array();
			$aNodeFlagPHPName = projectTo(NodeFlag::getAll(), 'sPHPName');
			foreach ($aDocument as $document) {
				$aFlag = array();
				foreach ($aNodeFlagPHPName as $iFlag => $sPHPName)
					$aFlag[$iFlag] = $document->$sPHPName;
				
				$sURL = $document->getAbsoluteURL();
				if (!$sURL)
					$sURL = $document->sPathPart;
				
				ksort($document->aidHuman);
				$aDocumentScript[] = array(
					$document->id,
					trim($document->sTitle),
					trim($sURL),
					$aFlag,
					array_values($document->aidHuman),
				);
			}
			$sScript .= html::jscriptSetVar('docs', $aDocumentScript);
			
			
			// Give parent properties and flags data: inherited value, full name, abbreviation letter
			$aParentData = array(
				'idNode'   => $this->id,
				'iWebMode' => $this->iWebMode,
				'aFlag'    => array(),
			);
			foreach ($node->parent->getNodeFlags() as $nodeFlag)
				$aParentData['aFlag'][] = getObjectFields($nodeFlag, getList('aValue;aImage'));
			
			$sScript .= html::jscriptSetVar('parentData', $aParentData);
			
			$s .=
				html::jscriptInline($sScript) .
				html::jscriptLink('global.js') .
				html::jscriptLink('edit_docsleft.js');
		}
		
		return $s;
	}
	
	
	protected function body()
	{
		if (!$this->wantEditing(false))
			return parent::body();
		
		$sError = $this->getError();
		if ($sError)
			return $sError;
		
		return ($this->bPaneLeft)
			? $this->contents()
			: $this->contentsFrameset();
	}
	
	
	protected function contentsFrameset()
	{
		$sURL = sDefaultURL;
		if (!bWebPaneLocalhost)
			foreach ($this->aAncestor as $nodeAncestor)
				if ($nodeAncestor->sURL)
					$sURL = $nodeAncestor->sURL;
		
		$target = new target('edit_web.php', array('sURL' => $sURL, 'iWebMode' => $this->iWebMode));
		
		return htmlframeset::frameset(
			htmlframeset::frame($this->getSelfTarget(targetParams, array('bPaneLeft' => true))->url(), 'frmedit') .
			htmlframeset::frame($target->url(), 'frmweb'), '', '50%,*');
	}
	
	
	private static function buttonJS($sAction, $sScript, $sToolTip, array $aAttr = array())
	{
		return form::button('', '', $sScript, $aAttr +
			attr::title($sToolTip) + attr::idClassNameSame($sAction));
	}
	
	private static function linkJS($sClassName, $sScript, $sLabel)
	{
		return html::link('#', $sLabel, array('onclick' => $sScript) + attr::className($sClassName));
	}
	
	
	protected function contents()
	{
		$node = $this->node;
		$s = '';
		
		$targetExit = $this->targetFromURL($this->sExitURL);
		
		// Ancestors
		$aTitlePart = array();
		foreach ($this->aAncestor as $nodeAncestor)
			$aTitlePart[] = $nodeAncestor->getLinkTitle();
		$s .= html::title(3, concat(html::map('text', $aTitlePart), '  '),
			attr::className('doctitle'));
		
		$s .= html::div(concat(array(
				self::buttonJS('exit', 'window.top.location.href=' . html::jscriptString($targetExit->url()) . ';', 'Sortir en annulant les modifications'),
				self::buttonJS('reload', 'location.reload()', 'Recharger la page en annulant les modifications'),
				self::buttonJS('docsSave', 'onDocsSave(false)', 'Enregistrer les modifications'),
				self::buttonJS('docsSaveExit', 'onDocsSave(true)', 'Enregistrer les modifications puis sortir'),
				self::linkJS('hideHumans', 'showHumans(false)', 'Masquer les auteurs') .
				self::linkJS('showHumans', 'showHumans(true)', 'Afficher les auteurs'),
				html::span('Enregistrement en cours', attr::className('docsSavingMessage')),
			), ' '),
			attr::idClassNameSame('docjsbar'));
		
		// Separator template
		$s .=
			html::div(
				self::actionLink('radio-off', 'Ajouter les documents  partir de ce point', 'selector') .
				self::actionLink('add',       'Ajouter un document ici'),
				attr::idClassName('tdocsep', 'docsep'));
		
		// Document template
		$aButton = array(
			self::actionLink('delete', '', 'askdelete'),
			self::actionLink('up'),
			self::actionLink('down'),
		);
		$sFlags = '';
		foreach (NodeFlag::getAll() as $iFlag => $nodeFlag)
			$sFlags .= self::actionLink($nodeFlag->sImagePrefix, '', 'flag' . $iFlag) . html::abbr('', '', attr::idClassName('flagAbbr' . $iFlag, 'flagAbbr'));
		$sDocument = html::brize(
			html::div(concat($aButton), attr::className('docbuttons')) .
			form::label('sTitle',  'Titre&nbsp;:') . form::text('IDsTitle', '', 50, maxLengthTitle, attr::className('autosize')),
			form::label('sURL',    'URL&nbsp;:')   . form::text('IDsURL',   '', 50, maxLengthURL, attr::className('autosize')) .
				' ' . DenifLink::createAction('#', 'earth', 'Tester le lien')->html(attr::id('urllink')) . $sFlags,
			html::span(form::label('sHuman', 'Auteurs&nbsp;:') . form::text('IDsHuman', '', 30) .
				' ' . html::span('', attr::id('humans')),
				attr::className('humansbar')));
		$s .= html::div(
			html::div(form::formJScript('return false;', $sDocument),
				attr::idClassName('tdoc', 'doc')),
			attr::id('list'));
		
		
		// Humans auto-completion menu
		$s .= html::div(form::button('createHuman', 'Crer un nouvel auteur...', 'onCreateHuman()'),
			attr::idClassName('humansPopup', 'popup'));
		
		// Human creation form
		$sCreateHuman =
			html::div('Crer un auteur', attr::className('title')) .
			form::label('sFullName', 'Prnom et Nom&nbsp;:') . form::text('IDsFullName', '', 50, maxLengthTitle) .
			form::label('sURL',      'Page perso&nbsp;:')    . form::text('IDsURL',      '', 50, maxLengthURL) .
			form::label('sEmail',    'E-mail&nbsp;:')        . form::text('IDsEmail',    '', 50, maxLengthEmail) .
			html::div(form::submit('', 'Crer') . form::button('', 'Annuler', 'onCreateHumanSubmit(false)'), attr::className('buttons'));
		$s .= html::div(form::formJScript('onCreateHumanSubmit(true)',
			$sCreateHuman, attr::id('fcreateHumanPopup')), attr::idClassName('createHumanPopup', 'popup'));
		
		return $s;
	}
	
	
	private static function actionLink($sAction, $sToolTip = '', $sId = '')
	{
		if (!$sId)
			$sId = $sAction;
		return DenifLink::createAction('#', $sAction, $sToolTip)->html(attr::id($sId));
	}
	
	
	/// @brief Check the sanity of the given command.
	/// Return an array of error strings.
	private function checkCommand(node $node, array $aChild, ObjectSafe $command)
	{
		$bAdmin = $this->isAdmin();
		
		$command->prepare('aHumanAdd;aDocKeep;aDocDelete');
		
		$aError = array();
		
		// Prepare the added humans: save the list to $command->aHumanAddOK
		$command->aHumanAddOK = array();
		foreach ($command->aHumanAdd as $human) {
			$human = new ObjectSafe($human);
			$human->prepare('id;sName;sURL;sEmail');
			
			$idOld = $human->id;
			if ($idOld < 0 && !isset($command->aHumanAddOK[$idOld])) {
				if (strlen($human->sName)  <= maxLengthTitle &&
				    strlen($human->sURL)   <= maxLengthURL &&
				    strlen($human->sEmail) <= maxLengthEmail) {
					$human->bValidated = $bAdmin;
					$command->aHumanAddOK[$idOLD] = $human;
				}else
					$aError[] = 'Auteur "' . $human->sName . '" : donnes invalides.';
			}
		}
		
		// Prepare the modified/created documents: save the list to $command->aDocKeepOK
		// Modified documents are first put to $aDocModified, indexed by their ID,
		// then we add to $command->aDocKeepOK the elements of $aDocModified
		// that effectively are children of the edited node.
		$command->aDocKeepOK = array();
		$aNodeFlag = NodeFlag::getAll();
		$sTable = node::$table->sName;
		$iOrder = -1;
		$idNew = 0;
		foreach ($command->aDocKeep as $doc) {
			$iOrder++;
			if (!is_array($doc) || count($doc) != 5) {
				$aError[] = 'Document : nombre darguments incorrect.';
				continue;
			}
			
			$data = ObjectSafe::fromArray($doc, 'id;sTitle;sURL;aFlag;aidHuman');
			$aData = $data->getFields('sTitle;sURL');
			$aData['iOrder'] = $iOrder;
			
			$bOK =
				strlen($data->sTitle) <= maxLengthTitle &&
				strlen($data->sURL)   <= maxLengthURL;
			foreach ($aNodeFlag as $iFlag => $nodeFlag) {
				$iValue = intval(defaultElement($data->aFlag, $iFlag, 0));
				if (!isDefineInRange($iValue, $nodeFlag->sShortName)) {
					$bOK = false;
					break;
				}
				$aData[$nodeFlag->sPHPName] = $iValue;
			}
			
			if (!$bOK) {
				$aError[] = 'Document "' . $data->sTitle . '" : donnes invalides.';
				continue;
			}
			
			$id = $data->id;
			if ($id > 0) {
				// The document existed before: update it
				
				$data->aData = $aData;
				$aDocModified[$id] = $data;
				
			}else{
				// New document
				
				$aData['dtCreation'] = time();
				$aData['iNode']      = nodeDocument;
				$aData['idParent']   = $this->id;
				$data->aData = $aData;
				$command->aDocKeepOK[] = $data;
			}
		}
		
		// Add to $command->aDocKeepOK the $aDocModified nodes
		// that are children of the edited node.
		foreach ($aChild as $nodeChild)
			if (isset($aDocModified[$nodeChild->id]))
				$command->aDocKeepOK[] = $aDocModified[$nodeChild->id];
		
		return $aError;
	}
	
	
	private function processCommand(node $node, ObjectSafe $command)
	{
		$aChild = bijectiveBy($node->loadChildren(), 'id');
		
		$aidNewDoc   = array();
		$aidNewHuman = array();
		$aError      = $this->checkCommand($node, $aChild, $command);
		$bOK         = false;
		
		if (!$aError) {
			
			// Get the human links
			$aDocumentHumans = array();
			foreach ($node->getLinksRecurse(nodeDocGroup, nodeDocument) as $link)
				if ($link->iProperty == propertyHuman)
					$aDocumentHumans[$link->idNode][$link->idProperty] = $link->id;
			
			// Create added humans
			if ($command->aHumanAddOK) {
				$sTable = ObjectTypeInfo::get(propertyHuman)->table->sName;
				$aHumanField = getList('sName;sURL;sEmail;bValidated');
				foreach ($command->aHumanAddOK as $human)
					$aidNewHuman[$human->id] = sql::insertIntoObj($sTable, $human, $aHumanField);
			}
			
			// Create/update/delete documents
			sql::lockTables(array(node::$table->sName, 'link'));
			
			$aHumanAddAll = array();
			$aHumanDeleteAll = array();
			
			$aURLNoDownloadExt = array_flip(getList(sURLNoDownloadExts));
			
			$aBrotherPathPart = array();
			
			$sTable = node::$table->sName;
			foreach ($command->aDocKeepOK as $doc) {
				
				// Map the negative human IDs to the matching real human ID
				foreach ($doc->aidHuman as &$idHuman) {
					$idHuman = intval($idHuman);
					if (isset($aidNewHuman[$idHuman]))
						$idHuman = $aidNewHuman[$idHuman];
				}
				unset($idHuman);
				
				$aHumanNew = array_flip($doc->aidHuman);
				
				$aData = $doc->aData;
				$id = $doc->id;
				
				// Take care of the path part
				if ($id > 0) {
					$nodeChild = $aChild[$id];
					$nodeChild->sURL = $aData['sURL'];
					$sPathPartError = 'le nom du fichier de cache na pas t chang';
					
				}else{
					$nodeChild = new node;
					foreach ($aData as $sMember => $value)
						$nodeChild->$sMember = $value;
					$sPathPartError = 'erreur  la validation du nom de ficher de cache';
				}
				
				if ($nodeChild->setPathPartFromURL($this->aAncestor, $aBrotherPathPart)) {
					$aData['sURL']      = $nodeChild->sURL;
					$aData['sPathPart'] = $nodeChild->sPathPart;
					$aBrotherPathPart[] = $nodeChild->sPathPart;
				}else
					$aError[] = 'Document "' . $doc->sTitle . '" : ' . $sPathPartError . '.';
				
				
				if ($id > 0) {
					// The document existed before: update it
					
					sql::update($sTable, $aData, sql::whereId($id));
					
					$aHumanOld    = defaultElement($aDocumentHumans, $id, array());
					$aHumanDelete = array_diff_key($aHumanOld, $aHumanNew);
					$aHumanAdd    = array_diff_key($aHumanNew, $aHumanOld);
					
					append($aHumanDeleteAll, $aHumanDelete);
					
				}else{
					// New document
					
					$target = target::fromURL($aData['sURL']);
					
					$iPingDownload = pdlDownload;
					if (!$target->getParam(sParamPrefix . 'bForceDownload', false)) {
						$sExt = path::getExtension($target->sPath);
						if (isset($aURLNoDownloadExt[$sExt]))
							$iPingDownload = pdlIgnore;
					}
					$aData['iPingDownload'] = $iPingDownload;
					$aData['sDetails']  = '';
					
					$id = $aidNewDoc[] = sql::insertInto($sTable, $aData);
					$aHumanAdd = $aHumanNew;
				}
				
				foreach ($aHumanAdd as $idHuman => $value)
					$aHumanAddAll[] = array($id, propertyHuman, $idHuman);
			}
			
			// Create/delete the human links
			if ($aHumanAddAll)
				sql::insertIntoMultiple('link', getList('idNode;iProperty;idProperty'), $aHumanAddAll);
			if ($aHumanDeleteAll)
				sql::delete('link', 'WHERE ' . sql::formatInIntList('id', $aHumanDeleteAll));
			
			// Delete the removed documents
			settype($command->aDocDelete, 'array');
			if ($command->aDocDelete) {
				$aidDocDelete = map('intval', $command->aDocDelete);
				sql::delete($sTable, sql::whereOne('idParent', $this->id) .
					' AND ' . sql::formatInIntList('id', $aidDocDelete));
				sql::delete('link', 'WHERE ' . sql::formatInIntList('idNode', $aidDocDelete));
			}
			
			sql::unlockTables();
			
			$bOK = true;
		}
		
		foreach ($aError as &$sError)
			$sError = str_replace('', '\'', $sError);
		
		JSON::output(array(
			'aidNewDoc'   => $aidNewDoc,
			'aidNewHuman' => $aidNewHuman,
			'aError'      => $aError,
			'bOK'         => $bOK,
		));
	}
}

Page::output(new PageFormEditDocs);

?>