<?php
// This file is part of DENIF, distributed under license GPLv3.
// Copyright 2006 Guillaume Ryder, Arthur Charguraud

require_once 'shared.inc.php';


class FormatInfo
{
	public $sExtensions, $sName, $sWindows, $sLinux;
	
	
	public function __construct($sExtensions, $sName, $sWindows, $sLinux = null)
	{
		if (!$sLinux)
			$sLinux = $sWindows;
		
		foreach (getList('sExtensions;sName;sWindows;sLinux') as $sMember)
			$this->$sMember = $$sMember;
	}
}


class PageFormats extends PageDenif
{
	private $aFormatInfo;
	
	
	protected function getTitle() { return 'Lire, convertir et imprimer les diffrents formats de fichiers'; }
	
	
	protected function contents()
	{
		$this->loadFormatInfos();
		
		$s = '';
		
		$s .= $this->getFormatInfos('sWindows', 'Windows',
			array(
				'PDF seulement&nbsp;: les options dimpression dAdobe Reader permettent dimprimer plusieurs pages par feuille.',
				'La plupart des imprimantes rcentes peuvent imprimer directement en <i>N</i> pages par feuille. Regarder dans les options de limprimante, accessibles  partir de la bote de dialogue dimpression.',
				'Utiliser ' . $this->linkOut('http://sourceforge.net/projects/pdfcreator/', 'PDF Creator') . ' pour convertir le document en PDF, en activant le mode plusieurs pages par feuille dans les options de PDF Creator.',
			),
			
			array('',
				array(
					'PDF seulement&nbsp;: utiliser ' . $this->linkOut('http://www.foolabs.com/xpdf/download.html', html::kbd('pdftops')) . ', dont voici les ' . $this->linkOut('ftp://ftp.foolabs.com/pub/xpdf/xpdf-3.02-win32.zip', 'binaires') . ' (et non pas ' . html::kbd('pdf2ps') . ' qui dgraderait sensiblement la qualit du PDF). Ligne de commande&nbsp;:' . html::br() . html::kbd('pdftops source.pdf'),
					'Utiliser PDF Creator pour convertir le document en PDF, puis ' . html::kbd('pdftops') . ' comme indiqu ci-dessus',
					'Utiliser limprimante virtuelle PostScript dAdobe, similaire  PDF Creator&nbsp;: ' . $this->linkOut('http://www.adobe.com/support/downloads/detail.jsp?ftpID=1500', 'Virtual PS Printer'),
				),
				
				' de ' . $this->linkOut('http://www.miktex.org/', 'MiKTeX'),
				
				html::listU(array(
					'Imprimer le fichier avec Ghostview vers PDF Creator',
					'Utiliser ' . html::kbd('ps2pdf') . ' fourni avec ' . $this->linkOut('http://www.miktex.org/', 'MiKTeX') . '&nbsp;: ' . html::kbd('ps2pdf -sPAPERSIZE#a4 <i>source.ps</i>'),
				)),
				
				html::par('Pour automatiser cette procdure, vous pouvez tlcharger les fichiers Batch ' . $this->linkOut('scripts/pdf_zoom_letter.bat', html::kbd('pdf_zoom_letter.bat')) . ' et ' . $this->linkOut('scripts/pdf_zoom_a4.bat', html::kbd('pdf_zoom_a4.bat')) . '. Sous rserve dditer les scripts en rajoutant le chemin daccs des programmes ' . html::kbd('pdftops') . ', ' . html::kbd('psnup') . ' et ' . html::kbd('ps2pdf') . ', il vous suffira de faire glisser votre fichier PDF sur le script pour le convertir. Le fichier original nest pas cras.'),
			));
		
		$s .= $this->getFormatInfos('sLinux', 'Linux et assimilables',
			array(
				'PDF seulement&nbsp;: les options dimpression dAdobe Reader permettent dimprimer plusieurs pages par feuille.',
				'La plupart des imprimantes rcentes peuvent imprimer directement en <i>N</i> pages par feuille. Utiliser le programme ' . html::kbd('xpp') . ' pour imprimer le fichier en dfinissant les options dimpression.',
				'Lutilitaire ' . html::kbd('psnup') . ' permet de modifier un fichier PostScript pour rassembler plusieurs pages sur chaque feuille. Ligne de commande&nbsp;:' . html::br() . html::kbd('psnup -<i>nb_pages_par_feuille source.ps dest.ps</i>'),
				'Lutilitaire ' . html::kbd('pdfnup') . ' (paquet ' . html::kbd('pdfjam') . ') permet de modifier un fichier PDF pour rassembler plusieurs pages sur chaque feuille. Ligne de commande (pour par exemple 2&nbsp;&times;&nbsp;4 pages par feuille)&nbsp;:' . html::br() .
				  html::kbd('pdfnup --nup 2x4 <i>source.pdf</i>') . html::br () .
				  html::kbd('pdfjam') . ' permet beaucoup dautres choses, notamment de rduire les marges, voir ci-dessous.',
			),
			
			array(html::par('Pour rduire les marges dun document PDF, utiliser loption ' . html::kbd('--trim') . ' de ' . html::kbd('pdfnup') . '&nbsp;:' . html::br() . html::kbd('pdfnup --trim "2cm 2cm 2cm 2cm" <i>source.pdf</i>') . html::br() . 'Lordre est gauche, bas, droite, haut. Cette option est bien sr combinable  loption ' . html::kbd('--nup') . '. Pour faire des essais, loption ' . html::kbd('-remote') . ' de ' . html::kbd('xpdf') . ' peut tre pratique (voir le man).'),
				array(
					'PDF seulement&nbsp;: utiliser ' . html::kbd('pdftops') . ' (et non pas ' . html::kbd('pdf2ps') . ' qui dgraderait sensiblement la qualit du PDF). Ligne de commande&nbsp;:' . html::br() . html::kbd('pdftops source.pdf'),
					'Utiliser les options dimpression du logiciel pour gnrer un fichier PostScript au lieu dimprimer.',
				),
				
				'',
				
				html::kbd('ps2pdf -sPAPERSIZE=a4 <i>source.ps</i>'),
				
				html::par('Pour automatiser cette procdure, vous pouvez tlcharger les scripts ' . $this->linkOut('scripts/pdf_zoom_letter.sh', html::kbd('pdf_zoom_letter.sh')) . ' et ' . $this->linkOut('scripts/pdf_zoom_a4.sh', html::kbd('pdf_zoom_a4.sh')) . '. Il suffit de leur donner en argument le nom du fichier PDF  zoomer. Le fichier original nest pas cras.'),
			));
		
		return $s;
	}
	
	
	private function loadFormatInfos()
	{
		$s7Zip    = $this->linkOut('http://www.7-zip.org/', '7-zip');
		$sAcrobat = $this->linkOut('http://www.adobe.com/products/acrobat/', 'Adobe Reader');
		$sGhost   = $this->linkOut('http://www.cs.wisc.edu/~ghost/', 'Ghostscript + Ghostview');
		$this->aFormatInfo = array(
			new FormatInfo(html::kbd('.pdf'), 'PDF (Portable Document Format)', $sAcrobat,
				$sAcrobat . ' (paquet ' . html::kbd('acroread') . '), ' . html::kbd('xpdf')),
			new FormatInfo(html::kbd('.ps'), 'PostScript', $sGhost, html::kbd('gv') . ', ' . html::kbd('ggv')),
			new FormatInfo(html::kbd('.zip'), 'Compress Zip', 'Explorateur Windows, ' . $s7Zip . ', Winzip, Winrar', html::kbd('unzip')),
			new FormatInfo(html::kbd('.ps.gz') . ' ou ' . html::kbd('ps.bz2'), 'PostScript compress',
				'Ouvrir directement avec Ghostview,' . html::br() . 'ou bien dcompresser dabord avec ' . $s7Zip,
				'Dcompresser avec ' . $this->linkOut('http://www.gzip.org/#exe', 'gunzip') . ' ou ' .
				$this->linkOut('http://www.bzip.org/downloads.html', 'bunzip2') . ' (paquet '. html::kbd('bzip2') .')' .
			  html::br() . 'puis lire comme un fichier PostScript'),
			new FormatInfo(html::kbd('.mws'), 'Feuille Maple', 'Maple'),
			new FormatInfo(html::kbd('.v'), 'Preuve ' . $this->linkOut('http://coq.inria.fr/', 'COQ'), 'diteur de texte, CoqIde'),
			new FormatInfo(html::kbd('.pl'), 'Source Prolog', 'diteur de texte'),
		);
	}
	
	
	private function getFormatInfos($sField, $sName, array $aPrinting, array $aZooming)
	{
		$s = $this->putTitle('Sous ' . $sName, strtolower(substr($sField, 1)));
		
		$s .= $this->putTitle('Lire les fichiers', '', 1);
		$aRow = array();
		foreach ($this->aFormatInfo as $formatInfo)
			$aRow[] = array($formatInfo->sExtensions, $formatInfo->sName, $formatInfo->$sField);
		$s .= html::table($aRow, '', attr::className('border'),
			html::trthList(getList('Extension;Format;Logiciel')));
		
		$s .= $this->putTitle('Imprimer plusieurs pages par feuille', '', 1);
		$s .= html::listU($aPrinting);
		
		$s .= $this->putTitle('Rduire les marges', '', 1);
		$s .=
			html::par('La plupart des polycopis sont gnrs avec des marges gigantesques, alors que lon pourrait les utiliser pour agrandir le texte lorsque lon imprime plusieurs pages par feuille. De plus, certains polycopis sont gnrs au format de papier amricain letter qui est plus large et moins haut que le A4, do perte de place supplmentaire  limpression.') .
			$aZooming[0] .
			html::listO(array(
				html::b('Convertir le document en PostScript') .
				html::listU($aZooming[1]),
				
				html::b('Recadrer les pages dun fichier PostScript') . html::br() .
				'Lutilitaire ' . html::kbd('psnup') . $aZooming[2] . ' permet, entre autres, de zoomer un fichier PostScript et de modifier ses marges. La syntaxe gnrale est la suivante&nbsp;:' .
				html::div(html::kbd('psnup -pA4 -s<i>facteur_zoom</i> [-b<i>marge</i>] [-W<i>largeur_source</i> -H<i>hauteur_source</i>] <i>source.ps dest.ps</i>')) .
				'On zoome le document en donnant un facteur de zoom suprieur  1, une marge ngative, et hauteur de page source infrieure  la dimension relle.' .
				html::listU(array(
					'Si le document est au format amricain (215,9 &times; 279,4&nbsp;mm est indiqu dans Adobe Reader quand on dplace la souris dans le coin infrieur gauche)&nbsp;:' .
						html::div(html::kbd('psnup -pA4 -s1.1 -b-1cm -W8.5in -H10in <i>source.ps dest.ps</i>')) .
					'En principe on devrait indiquer ' . html::kbd('11in') . ' au lieu de ' . html::kbd('10in') . ' pour respecter les dimensions du format amricain, cependant ce rglage permet de corriger les marges. Le dcalage de '. html::kbd('-1cm') . ' centre la page aprs le zoom.',
					'Si le document est au format A4&nbsp;:' .
						html::div(html::kbd('psnup -pA4 -s1.18 -b-2cm -W21.5cm -H29.7cm <i>source.ps dest.ps</i>')),
				)),
				
				html::b('Convertir le fichier PostScript en PDF (facultatif)') . html::br() .
				$aZooming[3],
			)) .
			$aZooming[4];
		
		return $s;
	}
}

Page::output(new PageFormats);

?>