<?php
// This file is part of DENIF, distributed under license GPLv3.
// Copyright 2006 Guillaume Ryder, Arthur Charguraud

require_once 'shared.inc.php';


class PageZip extends PageDenif
{
	public $idNode;
	private $node, $bAdmin, $aDocument, $iTotalSize;
	
	
	protected function getParameters() { return concatLists(parent::getParameters(), 'idNode'); }
	
	protected function getTitle() { return 'Tlchargement de plusieurs documents'; }
	
	
	protected function initialize()
	{
		parent::initialize();
		$this->loadParameters();
		
		if (!class_exists('ZipArchive'))
			$this->fatalError('Le module de cration darchives Zip nest pas activ.');
		
		$node = $this->node = getNode($this->idNode);
		if (!$node)
			$this->fatalError('Nud introuvable.');
		
		$this->bAdmin = $this->isAdmin();
		$this->aDocument = array();
		$this->iTotalSize = 0;
		$node->loadParentsChildrenRecurse();
		$this->getAccessibleDocuments($node);
	}
	
	/// @brief Gather all accessible cached documents in the given node, recursively.
	/// If the user is admin, do not take the rights into account.
	/// The documents are added to the #$aDocument array.
	private function getAccessibleDocuments(node $node)
	{
		if ($node->iNode == nodeDocument && $node->sPathPart) {
			$iSize = @filesize($node->sPathFull);
			if ($iSize !== false && is_file($node->sPathFull) &&
			   ($this->bAdmin || $node->canGuestAccessCache())) {
				$this->iTotalSize += $iSize;
				$this->aDocument[] = $node;
			}
		}else
			foreach ($node->aChild as $nodeChild)
				$this->getAccessibleDocuments($nodeChild);
	}
	
	
	protected function contents()
	{
		if ($this->aDocument && ($this->bAdmin || $this->iTotalSize <= maxZipTotalSize)) {
			
			// Create a temporary file for the archive;
			// delete it because ZipArchive does not consider
			// that empty files are valid empty archives
			$sZipFile = path::absolute(path::getTempFileName(sTempFilePrefix));
			@unlink($sZipFile);
			path::cleanTempFiles(sTempFilePrefix, path::dirname($sZipFile));
			
			// Open the archive
			$zip = new ZipArchive();
			if ($zip->open($sZipFile, ZipArchive::CREATE) !== true)
				$this->fatalError('Impossible de crer larchive Zip.');
			
			// Compress the files
			// Use node::formatZipTitle() to get the Zip file names
			$lenDirPrefix = strlen($this->node->sPathFull);
			foreach ($this->aDocument as $doc) {
				set_time_limit(30);
				$sDir = dirname(trim(substr($doc->sPathFull, $lenDirPrefix), '/\\'));
				if ($sDir == '.')
					$sDir = '';
				$sFile = path::concat($sDir, $doc->formatZipTitle());
				$zip->addFile($doc->sPathFull, iconv(sEncoding, 'cp850//TRANSLIT', $sFile));
			}
			$zip->close();
			
			// Send the file and quit
			$sNodePathRel = substr($this->node->sPathFull, strlen(sDataPath));
			$sZipFileName = 'denif_' . str_replace('/', '_', $sNodePathRel) . '.zip';
			self::uploadFile($sZipFile, 'Content-Type: application/zip', $sZipFileName);
			@unlink($sZipFile);
			die;
		}
		
		
		$s = $this->putTitle($this->getTitle());
		
		$s .= html::par('Impossible de crer une archive Zip contenant les documents de la ressource choisie, car la ressource ne contient aucun fichier mis en cache.');
		
		return $s;
	}
}

Page::output(new PageZip);

?>